from datetime import datetime, timedelta, date, time
import time as time_module


SECONDS_PER_DAY = int(timedelta(days=1).total_seconds())
SECONDS_PER_HOUR = int(timedelta(hours=1).total_seconds())
SECONDS_PER_MINUTE = int(timedelta(minutes=1).total_seconds())


def to_timestamp(val: datetime, unit="second"):
    if not val:
        return None
    return int(val.timestamp() * 1000) if unit == "millisecond" else int(val.timestamp())


def strftime(t: datetime | None = None, f="%Y-%m-%d %H:%M:%S"):
    if not t:
        t = datetime.now()
    return t.strftime(f)


def strptime(date_string, f="%Y-%m-%d %H:%M:%S"):
    return datetime.strptime(date_string, f)


def get_end_time_of_day(day_delta=0):
    return datetime.combine(date.today(), time.max) + timedelta(days=day_delta)


def get_begin_time_of_day(day_delta=0):
    return datetime.combine(date.today(), time.min) + timedelta(days=day_delta)


def current_milli_time():
    return round(time_module.time() * 1000)


def current_sec_time():
    return round(time_module.time())


# string float
def fromtimestamp_in_milli(t) -> datetime:
    if not isinstance(t, int):
        t = int(t)
    return datetime.fromtimestamp(t // 1000)


def fromtimestamp_in_sec(t) -> datetime:
    if not isinstance(t, int):
        t = int(t)
    return datetime.fromtimestamp(t)



# # celery service management

# import os
# from os import path
# import subprocess
# from enum import Enum
# from dotenv import load_dotenv

# import typer

# load_dotenv()  # # take environment variables from .env.
# PROJECT_NAME = os.getenv("PROJECT_NAME", "")
# PROJECT_DIR = os.getenv("PROJECT_DIR", "")
# LOG_DIR = os.getenv("LOG_DIR", "")

# CELERY_WORKER_SYSTEMD_TEMPLATE = """[Unit]
# Description={NAME}.worker.service
# ConditionPathExists={CELERY_PATH}
# After=network.target

# [Service]
# Environment="PROMETHEUS_MULTIPROC_DIR={PROMETHEUS_MULTIPROC_DIR}"
# WorkingDirectory={WORKING_DIRECTORY}
# ExecStart={CELERY_PATH} --app={NAME}.tasks worker --pool=gevent --loglevel=INFO --concurrency=1
# ExecReload=kill -HUP $MAINPID
# RestartSec=1
# Restart=always

# [Install]
# WantedBy=multi-user.target"""


# CELERY_BEAT_SYSTEMD_TEMPLATE = """[Unit]
# Description={NAME}.beat.service
# ConditionPathExists={CELERY_PATH}
# After=network.target

# [Service]
# Environment="PROMETHEUS_MULTIPROC_DIR={PROMETHEUS_MULTIPROC_DIR}"
# WorkingDirectory={WORKING_DIRECTORY}
# ExecStart={CELERY_PATH} --app={NAME}.tasks beat --loglevel=INFO
# ExecReload=kill -HUP $MAINPID
# RestartSec=1
# Restart=always

# [Install]
# WantedBy=multi-user.target"""


# def run_command(command) -> tuple[str, str, int]:
#     result = subprocess.run(command, shell=True, capture_output=True, text=True, check=False, executable="/bin/bash")
#     return result.stdout.strip(), result.stderr.strip(), result.returncode


# def echo_succ(content):  # 绿色
#     typer.secho(content, fg=typer.colors.GREEN)


# def echo_err(content):  # 红色
#     typer.secho(content, fg=typer.colors.RED)


# def get_service_state(service_name: str):
#     state, _, _ = run_command(f"systemctl is-active {service_name}")
#     return state


# class CeleryService(Enum):
#     WORKER = "worker"
#     BEAT = "beat"

#     def __str__(self):
#         return self.value


# def configure(service: CeleryService):
#     """configure service"""
#     typer.echo(f"PROJECT_NAME: {PROJECT_NAME}")
#     typer.echo(f"PROJECT_DIR: {PROJECT_DIR}")
#     typer.echo(f"LOG_DIR: {LOG_DIR}")

#     service_name = f"{PROJECT_NAME}.{service}"
#     service_path = f"/etc/systemd/system/{service_name}.service"
#     if path.exists(service_path):
#         echo_err(f"{service_path} already exists, delete it first if you want to reconfigure")
#         return

#     template = CELERY_WORKER_SYSTEMD_TEMPLATE if service == CeleryService.WORKER else CELERY_BEAT_SYSTEMD_TEMPLATE
#     celery_path = path.join(PROJECT_DIR, "venv/bin/celery")
#     with open(service_path, "w", encoding="utf-8") as f:
#         f.write(
#             template.format(
#                 NAME=PROJECT_NAME,
#                 CELERY_PATH=celery_path,
#                 WORKING_DIRECTORY=PROJECT_DIR,
#                 PROMETHEUS_MULTIPROC_DIR=path.join(LOG_DIR, "metrics"),
#             )
#         )
#     os.chmod(service_path, mode=0o755)

#     output, err, code = run_command(f"systemctl daemon-reload && systemctl enable {service_name}")
#     if code != 0:
#         echo_err(f"failed to enable service: {err}")
#         raise SystemExit
#     typer.echo(output)
#     echo_succ(f"{service_name} enabled")


# def start(service: CeleryService):
#     """start service"""
#     service_name = f"{PROJECT_NAME}.{service}"
#     if get_service_state(service_name) == "active":
#         echo_err(f"{service_name} is already running, use reload to restart it gracefully")
#         raise SystemExit

#     _, err, code = run_command(f"systemctl start {service_name}")
#     if code != 0:
#         echo_err(err)
#         raise SystemExit
#     echo_succ(f"{service_name} started")


# def stop(service: CeleryService):
#     """stop service"""
#     service_name = f"{PROJECT_NAME}.{service}"
#     if get_service_state(service_name) == "inactive":
#         echo_err(f"{service_name} is already stopped")
#         raise SystemExit

#     _, err, code = run_command(f"systemctl stop {service_name}")
#     if code != 0:
#         echo_err(err)
#         raise SystemExit
#     echo_succ(f"{service_name} stopped")


# def reload(service: CeleryService):
#     """reload service gracefully"""
#     service_name = f"{PROJECT_NAME}.{service}"
#     if get_service_state(service_name) == "inactive":
#         echo_err(f"{service_name} is not running, use start to start it")
#         raise SystemExit

#     _, err, code = run_command(f"systemctl reload {service_name}")
#     if code != 0:
#         echo_err(err)
#         raise SystemExit
#     echo_succ(f"{service_name} reloaded")


# def status(service: CeleryService):
#     """show service status"""
#     service_name = f"{PROJECT_NAME}.{service}"
#     output, err, _ = run_command(f"systemctl status {service_name}")
#     typer.echo(output)
#     echo_err(err)  # warning message


# def log(service: CeleryService):
#     """show service last 30 lines of log"""
#     service_name = f"{PROJECT_NAME}.{service}"
#     output, err, _ = run_command(f"journalctl -u {service_name} -n 30")
#     typer.echo(output)
#     echo_err(err)  # warning message


# def reload_celery_services():
#     """reload celery worker and beat gracefully"""
#     if get_service_state(f"{PROJECT_NAME}.{CeleryService.WORKER}") == "active":
#         reload(CeleryService.WORKER)
#     if get_service_state(f"{PROJECT_NAME}.{CeleryService.BEAT}") == "active" and typer.confirm("Do you want to reload beat?"):
#         reload(CeleryService.BEAT)


# worker = typer.Typer(help="celery worker management", no_args_is_help=True)
# worker.command("configure", help="configure celery worker")(lambda: configure(CeleryService.WORKER))
# worker.command("start", help="start celery worker")(lambda: start(CeleryService.WORKER))
# worker.command("stop", help="stop celery worker")(lambda: stop(CeleryService.WORKER))
# worker.command("reload", help="reload celery worker gracefully")(lambda: reload(CeleryService.WORKER))
# worker.command("status", help="show celery worker status")(lambda: status(CeleryService.WORKER))
# worker.command("log", help="show celery worker last 30 lines of log")(lambda: log(CeleryService.WORKER))

# beat = typer.Typer(help="celery beat management", no_args_is_help=True)
# beat.command("configure", help="configure celery beat")(lambda: configure(CeleryService.BEAT))
# beat.command("start", help="start celery beat")(lambda: start(CeleryService.BEAT))
# beat.command("stop", help="stop celery beat")(lambda: stop(CeleryService.BEAT))
# beat.command("reload", help="reload celery beat gracefully")(lambda: reload(CeleryService.BEAT))
# beat.command("status", help="show celery beat status")(lambda: status(CeleryService.BEAT))
# beat.command("log", help="show celery beat last 30 lines of log")(lambda: log(CeleryService.BEAT))



"""
AES/CBC/PKCS7Padding(PKCS5Padding) encrypt and decrypt
"""

import base64

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import padding
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes


iv = bytes(16)  # iv is empty 16 bytes data


def encrypt(data: str | bytes, key: str):
    """encrypt data with AES/CBC/PKCS7Padding"""

    if isinstance(data, str):
        data = data.encode()

    encryptor = Cipher(algorithms.AES(key.encode()), modes.CBC(iv), backend=default_backend()).encryptor()
    padder = padding.PKCS7(algorithms.AES.block_size).padder()  # type: ignore
    padded_data = padder.update(data) + padder.finalize()
    cipher_text = encryptor.update(padded_data) + encryptor.finalize()
    return base64.b32encode(cipher_text).decode()


def decrypt(data: str | bytes, key: str):
    """decrypt data with AES/CBC/PKCS7Padding"""

    if isinstance(data, str):
        data = data.encode()

    decryptor = Cipher(algorithms.AES(key.encode()), modes.CBC(iv), backend=default_backend()).decryptor()
    decrypted_data = decryptor.update(base64.b32decode(data)) + decryptor.finalize()
    unpadder = padding.PKCS7(algorithms.AES.block_size).unpadder()  # type: ignore # PKCS7解填充
    unpadded_data = unpadder.update(decrypted_data) + unpadder.finalize()
    return unpadded_data.decode()



from enum import Enum


class MediaType(Enum):
    VIDEO = "video"
    AUDIO = "audio"
    IMAGE = "image"

    def __str__(self):
        return self.value


video_quality_note_dict = {4320: "8K", 2160: "4K", 1440: "2K"}


def get_quality_note_by_quality(quality: int):
    return video_quality_note_dict.get(quality, f"{quality}P")


extension_to_media_type_dict = {
    ".heic": MediaType.IMAGE,
    ".heif": MediaType.IMAGE,
    ".avif": MediaType.IMAGE,
}



import random

DEFAULT = "Mozilla/5.0"

_pc = [
    "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/70.0.3538.77 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/55.0.2883.87 Safari/537.36",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/54.0.2866.71 Safari/537.36",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/52.0.2762.73 Safari/537.36",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/49.0.2656.18 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:77.0) Gecko/20190101 Firefox/77.0",
    "Mozilla/5.0 (Windows NT 10.0; WOW64; rv:77.0) Gecko/20100101 Firefox/77.0",
    "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:39.0) Gecko/20100101 Firefox/75.0",
    "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10.10; rv:75.0) Gecko/20100101 Firefox/75.0",
    "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/53.0.2785.104 Safari/537.36 Core/1.53.2759.400 QQBrowser/9.6.11266.400",
]

_mobile = [
    "Mozilla/5.0 (iPhone; CPU iPhone OS 10_3_3 like Mac OS X) AppleWebKit/603.3.6 (KHTML, like Gecko) Mobile/14G5053a",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 9_3_2 like Mac OS X) AppleWebKit/601.1.46 (KHTML, like Gecko) Mobile/13F69",
    "Mozilla/5.0 (Linux; U; Android 6.0.1; zh-cn; OPPO A57 Build/MMB29M) AppleWebKit/533.1 (KHTML, like Gecko) Mobile Safari/533.1",
    "Mozilla/5.0 (Linux; U; Android 6.0.1; zh-cn; OPPO R9s Build/MMB29M) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/38.0.0.0 Mobile Safari/537.36 OppoBrowser/4.2.8",
    "Mozilla/5.0 (Linux; U; Android 7.0; zh-cn; HUAWEI NXT-AL10 Build/HUAWEINXT-AL10) AppleWebKit/537.36 (KHTML, like Gecko)Version/4.0 Chrome/37.0.0.0 MQQBrowser/7.6 Mobile Safari/537.36",
    "Mozilla/5.0 (Linux; U; Android 6.0.1;zh_cn; C106-9 Build/ZCXCNCT5801804271S) AppleWebKit/537.36 (KHTML, like Gecko)Version/4.0 Chrome/49.0.0.0 Mobile Safari/537.36 EUI Browser/5.8.018S",
    "Mozilla/5.0 (Linux; U; Android 2.x; zh-cn;-) UC AppleWebkit(like Gecko) Mobile Safari/530",
    "Mozilla/5.0 (Linux; U; Android 4.4.2; zh-CN; PE-TL20 Build/HuaweiPE-TL20) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/40.0.2214.89 UCBrowser/11.5.2.942 Mobile Safari/537.36",
    "Mozilla/5.0 (Linux; Android 6.0.1; vivo X9Plus Build/MMB29M; wv) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/53.0.2785.49 Mobile MQQBrowser/6.2 TBS/043305 Safari/537.36 MicroMessenger/6.5.6.1020 NetType/WIFI Language/zh_CN",
    "Mozilla/5.0 (Linux; U; Android 7.0; zh-CN; MI 5 Build/NRD90M) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/40.0.2214.89 UCBrowser/11.5.6.946 Mobile Safari/537.36",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 11_4_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/11.0 Mobile/15E148 Safari/604.1",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 12_0_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/12.0 Mobile/15E148 Safari/604.1",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 11_3 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/11.3 Mobile/15E302 Safari/604.1 RDDocuments/6.7.1.567",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 11_4_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/11.4 Mobile/15G77 Safari/",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 11_2_1 like Mac OS X) AppleWebKit/604.4.7 (KHTML, like Gecko) Version/11.2 Mobile/15C153 Safari/",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 12_0_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/16A404",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 11_3 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/11.3 Mobile/15E216 Safari/",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 12_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/12.0 Mobile/15E148 Safari/604.1",
    "Mozilla/5.0 (iPhone; CPU iPhone OS 10_3_3 like Mac OS X) AppleWebKit/603.3.8 (KHTML, like Gecko) Version/10.3 Mobile/14G60 Safari/",
]


def fixed_pc_ua():
    return _pc[0]


def random_pc_ua():
    return random.choice(_pc)


def fixed_mobile_ua():
    return _mobile[0]


def random_mobile_ua():
    return random.choice(_mobile)


def ua_headers(useragent=None):
    return {"User-Agent": DEFAULT if useragent is None else useragent}



from anyio import run_process


async def exec_shell(cmd):
    """异步执行shell命令，返回stdout和stderr，
    cmd可传数组或者字符串，如: ['ls', '-al'] 或 'ls -al'"""
    ret = await run_process(cmd, check=False)
    return ret.stdout.decode("utf-8").strip(), ret.stderr.decode("utf-8").strip()


def print_succ(content):  # 绿色
    print(f"\033[32m{content}\033[0m")


def print_err(content):  # 红色
    print(f"\033[31m{content}\033[0m")



def blur_phone(phone):
    """blur phone number"""
    if not phone or len(phone) < 11:
        return phone
    return phone[:3] + "****" + phone[7:]


def abbreviate(text, max_len=2, marker="..."):
    """slice text to max_len and add marker if text is too long"""
    return text[0:max_len] + marker if len(text) > max_len else text


def ensure_str(v: bytes | str):
    """convert bytes to str"""
    return v.decode() if isinstance(v, bytes) else v


def ensure_bytes(v: bytes | str):
    """convert str to bytes"""
    return v.encode() if isinstance(v, str) else v


def safe_int(v: str | int, default: int | None = None) -> int | None:
    try:
        return int(v)
    except (ValueError, TypeError, OverflowError):
        return default



from fastapi import FastAPI
from fastapi.encoders import jsonable_encoder
from fastapi.responses import JSONResponse
from fastapi.exceptions import RequestValidationError
from starlette.exceptions import HTTPException
from starlette.status import HTTP_422_UNPROCESSABLE_ENTITY


async def http_exception_handler(_request, exc: HTTPException):
    """Register it for Starlette's HTTPException. This way, if any part of Starlette's internal code,
    or a Starlette extension or plug-in, raises a Starlette HTTPException, your handler will be able to catch and handle it."""
    headers = getattr(exc, "headers", None)
    return JSONResponse(
        status_code=exc.status_code,
        content={
            "message": exc.detail,
        },
        headers=headers,
    )


async def validation_error_handler(_request, exc: RequestValidationError):
    return JSONResponse(
        status_code=HTTP_422_UNPROCESSABLE_ENTITY,
        content={
            "message": "Request Validation Error",
            "detail": jsonable_encoder(exc.errors()),
        },
    )


def install_exception_handlers(app: FastAPI):
    app.exception_handler(HTTPException)(http_exception_handler)
    app.exception_handler(RequestValidationError)(validation_error_handler)



import base64
import hashlib
from knifes.strings import ensure_bytes


def md5(data: bytes | str) -> str:
    return hashlib.md5(ensure_bytes(data)).hexdigest()


def b64encode(data: bytes | str) -> str:
    return base64.b64encode(ensure_bytes(data)).decode()


def b64decode(data: bytes | str) -> str:
    return base64.b64decode(ensure_bytes(data)).decode()


def b32encode(data: bytes | str) -> str:
    return base64.b32encode(ensure_bytes(data)).decode()


def b32decode(data: bytes | str) -> str:
    return base64.b32decode(ensure_bytes(data)).decode()



def safe_list_get(l_, *, idx=0, default=None):
    """get element from list safely"""
    try:
        if not isinstance(l_, list):
            raise TypeError
        return l_[idx]
    except (IndexError, TypeError):
        return default


def safe_list_index(l_, element, default=None):
    """get index of element from list safely"""
    try:
        if not isinstance(l_, list):
            raise TypeError
        return l_.index(element)
    except (ValueError, TypeError):
        return default



import pkgutil
from os import path


def get_all_module_name(pkg_path, prefix=""):
    """How To Use?
    import xx.abc
    get_all_module_name(os.path.dirname(xx.abc.__file__), prefix=xx.abc.__name__ + '.')
    """
    module_name_list = []
    for m in pkgutil.iter_modules([pkg_path], prefix=prefix):
        if m.ispkg:
            name = m.name.removeprefix(prefix)
            module_name_list.extend(get_all_module_name(path.join(pkg_path, name), prefix=prefix + name + "."))
        else:
            module_name_list.append(m.name)
    return module_name_list



from fastapi import Request


def get_client_ip(request: Request):
    x_forwarded_for = request.headers.get("X-Forwarded-For")
    if x_forwarded_for:
        return x_forwarded_for.split(",")[0].strip()
    return request.client.host  # type: ignore



# # deploy script for fastapi project

# import os
# import time
# from os import path
# from typing import Annotated

# import typer

# from knifes.deploy.celery import run_command, echo_err, echo_succ, PROJECT_NAME, PROJECT_DIR, LOG_DIR, get_service_state, worker, beat, reload_celery_services

# GUNICORN_SYSTEMD_TEMPLATE = """[Unit]
# Description={NAME}.service
# ConditionPathExists={GUNICORN_PATH}
# After=network.target

# [Service]
# Environment="PROMETHEUS_MULTIPROC_DIR={PROMETHEUS_MULTIPROC_DIR}"
# WorkingDirectory={WORKING_DIRECTORY}
# ExecStart={GUNICORN_PATH} -c {GUNICORN_CONF_PATH}
# ExecReload=kill -HUP $MAINPID
# RestartSec=1
# Restart=always

# [Install]
# WantedBy=multi-user.target"""

# app = typer.Typer(no_args_is_help=True)


# @app.callback()
# def callback():
#     """deploy script for fastapi project"""
#     if not PROJECT_NAME or not PROJECT_DIR or not LOG_DIR:
#         echo_err("please set PROJECT_NAME, PROJECT_DIR, LOG_DIR environment variables")
#         raise SystemExit


# app.add_typer(worker, name="worker", rich_help_panel="Celery")
# app.add_typer(beat, name="beat", rich_help_panel="Celery")


# @app.command(rich_help_panel="Project")
# def knifes():
#     """force install latest knifes"""
#     activate_path = path.join(PROJECT_DIR, "venv/bin/activate")
#     cmd = f"source {activate_path} && pip install knifes --index-url https://pypi.python.org/simple -U"
#     output, err, _ = run_command(cmd)
#     typer.echo(output)
#     echo_err(err)  # warning message


# @app.command(rich_help_panel="Project")
# def modules():
#     """install project dependencies"""
#     _install_modules()


# @app.command(rich_help_panel="Gunicorn")
# def log(line_count: Annotated[int, typer.Argument(help="number of lines to read")] = 10):
#     """read gunicorn last log"""
#     _read_last_log(line_count)


# @app.command(rich_help_panel="Gunicorn")
# def status():
#     """show gunicorn status"""
#     output, err, _ = run_command(f"systemctl status {PROJECT_NAME}")
#     typer.echo(output)
#     echo_err(err)  # warning message


# @app.command(rich_help_panel="Gunicorn")
# def configure():
#     """configure gunicorn service"""
#     typer.echo(f"PROJECT_NAME: {PROJECT_NAME}")
#     typer.echo(f"PROJECT_DIR: {PROJECT_DIR}")
#     typer.echo(f"LOG_DIR: {LOG_DIR}")

#     service_path = f"/etc/systemd/system/{PROJECT_NAME}.service"
#     if path.exists(service_path):
#         echo_err(f"{service_path} already exists, delete it first if you want to reconfigure")
#         return

#     with open(service_path, "w", encoding="utf-8") as f:
#         f.write(
#             GUNICORN_SYSTEMD_TEMPLATE.format(
#                 NAME=PROJECT_NAME,
#                 GUNICORN_PATH=path.join(PROJECT_DIR, "venv/bin/gunicorn"),
#                 GUNICORN_CONF_PATH=path.join(PROJECT_DIR, "gunicorn_conf.py"),
#                 WORKING_DIRECTORY=PROJECT_DIR,
#                 PROMETHEUS_MULTIPROC_DIR=path.join(LOG_DIR, "metrics"),
#             )
#         )
#     os.chmod(service_path, mode=0o755)

#     output, err, code = run_command(f"systemctl daemon-reload && systemctl enable {PROJECT_NAME}")
#     if code != 0:
#         echo_err(f"failed to enable service: {err}")
#         raise SystemExit
#     typer.echo(output)
#     echo_succ(f"{PROJECT_NAME} enabled")


# @app.command(rich_help_panel="Gunicorn")
# def start():
#     """start gunicorn"""
#     if get_service_state(PROJECT_NAME) == "active":
#         echo_err(f"{PROJECT_NAME} is already running, use reload to restart it gracefully")
#         raise SystemExit

#     _pull_latest_code()
#     _install_modules()  # install modules before starting gunicorn

#     _, err, code = run_command(f"systemctl start {PROJECT_NAME}")
#     if code != 0:
#         echo_err(err)
#         raise SystemExit
#     echo_succ(f"{PROJECT_NAME} started")

#     time.sleep(2)
#     _read_last_log()


# @app.command(rich_help_panel="Gunicorn")
# def reload():
#     """reload gunicorn gracefully"""
#     if get_service_state(PROJECT_NAME) == "inactive":
#         echo_err(f"{PROJECT_NAME} is not running, use start to start it")
#         raise SystemExit

#     _pull_latest_code()
#     _install_modules()

#     _, err, code = run_command(f"systemctl reload {PROJECT_NAME}")
#     if code != 0:
#         echo_err(err)
#         raise SystemExit
#     echo_succ(f"{PROJECT_NAME} reloaded")

#     # read last 6 lines of log
#     time.sleep(2)
#     _read_last_log()

#     # reload celery worker and beat
#     reload_celery_services()


# @app.command(rich_help_panel="Gunicorn")
# def stop():
#     """stop gunicorn"""
#     if get_service_state(PROJECT_NAME) == "inactive":
#         echo_err(f"{PROJECT_NAME} is already stopped")
#         raise SystemExit

#     _, err, code = run_command(f"systemctl stop {PROJECT_NAME}")
#     if code != 0:
#         echo_err(err)
#         raise SystemExit
#     echo_succ(f"{PROJECT_NAME} stopped")

#     # read last 6 lines of log
#     time.sleep(2)
#     _read_last_log()


# @app.command()
# def restart():
#     """restart gunicorn"""
#     if get_service_state(PROJECT_NAME) == "inactive":
#         echo_err(f"{PROJECT_NAME} is not running, use start to start it")
#         raise SystemExit

#     _, err, code = run_command(f"systemctl restart {PROJECT_NAME}")
#     if code != 0:
#         echo_err(err)
#         raise SystemExit
#     echo_succ(f"{PROJECT_NAME} restarted")

#     # read last 6 lines of log
#     time.sleep(2)
#     _read_last_log()


# def _pull_latest_code():
#     output, err, code = run_command(f"cd {PROJECT_DIR} && git pull origin main")
#     if code != 0:
#         echo_err(err)
#         raise SystemExit
#     typer.echo(output)
#     echo_err(err)  # warning message


# def _install_modules():
#     activate_path = path.join(PROJECT_DIR, "venv/bin/activate")
#     requirements_path = path.join(PROJECT_DIR, "requirements.txt")
#     cmd = f"source {activate_path} && pip install -r {requirements_path}"
#     output, err, code = run_command(cmd)
#     if code != 0:
#         echo_err(f"failed to install modules: {err}")
#         raise SystemExit
#     typer.echo(output)
#     echo_err(err)  # warning message


# def _read_last_log(line_count=10):
#     gunicorn_log_path = path.join(LOG_DIR, "gunicorn.log")
#     output, err, _ = run_command(f"tail -{line_count} {gunicorn_log_path}")
#     typer.echo(output)
#     echo_err(err)



from setuptools import setup, find_packages


setup(
    name="knifes",
    version="1.0.2",
    author="knifes",
    author_email="author@example.com",
    description="Swiss Army Knife",
    long_description="`pip install knifes --index-url https://pypi.python.org/simple -U`",
    long_description_content_type="text/markdown",
    url="https://github.com/",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=find_packages(),
    python_requires=">=3.10",
    install_requires=[
        "anyio",  # required by shell
        # "python-dotenv",  # required by deploy
        # "gevent",  # required by deploy/celery
        # "httpx[http2]",
        # 'cryptography',  #  required by aes
    ],
    # entry_points={
    #     "console_scripts": [
    #         "deploy = knifes.deploy.main:app",
    #     ],
    # },
)



import mimetypes
import os
from urllib import parse
from urllib.parse import urlparse

from knifes.media import MediaType, extension_to_media_type_dict


def get_url_from_text(text):
    if not text:
        return text
    start_index = text.rfind("http://")
    if start_index == -1:
        start_index = text.rfind("https://")
    if start_index == -1:
        return None
    text = text[start_index:]
    # 去掉空格后内容
    end_index = text.find(" ")
    if end_index != -1:
        text = text[0:end_index]
    return text


# m.oasis.weibo.cn => weibo.cn
def get_base_domain_with_suffix_from_url(url) -> str:
    """get base domain with suffix from url, like 'm.oasis.weibo.cn' => 'weibo.cn'"""
    if not url:
        return url
    u = parse.urlparse(url)
    if not u.hostname:
        return ""
    host_list = u.hostname.split(".")
    if len(host_list) < 2:
        return ""
    return host_list[-2].lower() + "." + host_list[-1].lower()


# m.oasis.weibo.cn => weibo
def get_base_domain_from_url(url) -> str:
    """get base domain from url, like 'm.oasis.weibo.cn' => 'weibo'"""
    if not url:
        return url
    u = parse.urlparse(url)
    if not u.hostname:
        return ""
    host_list = u.hostname.split(".")
    if len(host_list) < 2:
        return ""
    return host_list[-2].lower()


# m.oasis.weibo.cn => oasis.weibo
def get_sub_domain_from_url(url) -> str:
    """get sub domain from url, like 'm.oasis.weibo.cn' => 'oasis.weibo'"""
    if not url:
        return url
    u = parse.urlparse(url)
    if not u.hostname:
        return ""
    host_list = u.hostname.split(".")
    if len(host_list) < 3:
        return ""
    return host_list[-3].lower() + "." + host_list[-2].lower()


# 目前支持video、audio、image类型，其他返回None
# fixed bug for 'http://v.example.com/o0/000bH9u0lx07TQ0OHwmA01041200Onth0E010.mp4?label=mp4_hd&template=852x480.28.0&ori=0&ps=1CwnkDw1GXwCQx&Expires=1645114865&ssig=8QbZXUCE85&KID=unistore,video'
def guess_media_type(url) -> MediaType | None:
    url_without_query = url.split("?", maxsplit=1)[0]
    mimetype, _ = mimetypes.guess_type(url_without_query)
    if not mimetype:
        try:
            return extension_to_media_type_dict.get(get_extension(url_without_query.lower()))
        except:  # noqa: E722
            return None
    if mimetype in ("application/x-mpegurl", "application/vnd.apple.mpegurl"):  # .m3u8
        return MediaType.VIDEO
    try:
        return MediaType(mimetype.split("/")[0])
    except:  # noqa: E722
        return None


def parse_url(url) -> tuple[str, list, dict]:
    """return (hostname、path_list, query_dict)
    the difference between hostname and netloc is that hostname does not include the port number
    """
    parsed = parse.urlparse(url)
    return (
        parsed.hostname,
        parsed.path.strip("/").split("/"),
        dict(parse.parse_qsl(parsed.query)),
    )


def parse_path(url) -> list[str]:
    """return path list, like ['a', 'b', 'c']"""
    return parse.urlparse(url).path.strip("/").split("/")


def parse_query(url) -> dict[str, str]:
    """return query dict, like {'a': '1', 'b': '2'}"""
    return dict(parse.parse_qsl(parse.urlparse(url).query))


def get_query_value(url, parameter) -> str | None:
    """return query value by parameter, like 'http://example.com?a=1&b=2' => get_query_value(url, 'b') => '2'"""
    return parse_query(url).get(parameter)


def sanitize_url(url, scheme="http"):
    """Prepend protocol-less URLs with `http:` scheme in order to mitigate
    the number of unwanted failures due to missing protocol
    """
    if not url:
        return url
    if url.startswith("//"):
        return f"{scheme}:{url}"
    return url


def get_extension(url):
    return os.path.splitext(parse.urlparse(url).path)[1]


def is_http_url(url):
    result = urlparse(url)
    return all([result.scheme, result.netloc]) and result.scheme in {"http", "https"}
